import requests
import datetime


class CrowdTangle:
    """
    Classe que faz o request do endpoint e retorna um dataframe com as informações
    """
    def __init__(self, ct_token):
        self.today = datetime.datetime.now().date()
        self.ct_token = ct_token

    @staticmethod
    def create_payload_leader(account_ids=None, count=None, end_date=None, list_id=None, offset=None,
                              order_by=None, sort_by=None, start_date=None):
        """
        Cria o payload que será utilizado na request
        Args:
            account_ids: ids of accounts
            count: number of items returned
            end_date: end date of the search
            list_id: The list of the leaderboard to retrieve.
            offset: The number of rows to offset.
            order_by: the order of the sort. asc ou desc(ascendente ou descendente)
            sort_by: The method by which the accountStatistics are sorted.	total_interactions or interaction_rate
            start_date: The startDate of the leaderboard rage.

        Returns: Payload com as informações colocadas

        """
        payload = {}
        if account_ids is not None:
            payload['accountIds'] = account_ids
        if count is not None:
            payload['count'] = count
        if end_date is not None:
            payload['endDate'] = end_date
        if list_id is not None:
            payload['listId'] = list_id
        if offset is not None:
            payload['offset'] = offset
        if order_by is not None:
            payload['orderBy'] = order_by
        if sort_by is not None:
            payload['sortBy'] = sort_by
        if start_date is not None:
            payload['startDate'] = start_date
        return payload

    def create_payload_posts(self, accounts=None, branded_content=None, count=None, end_date=None, include_history=None,
                            language=None, list_ids=None, min_interactions=None, offset=None, page_admin_top_country=None,
                            search_term=None, sort_by=None, start_date=None, timeframe=None, types=None, verified=None):
        """
        Gera payload para posteriormente fazer o request do endpoint
        Args:
            accounts: String of nickcnames of accounts or id of list of account
            branded_content: Boolean to include or exclude branded content
            count: Number of items to return
            end_date: Data final que o post foi criado
            include_history: Boolean to include or exclude history data
            language: Code of language of post
            list_ids: Id that contains a set of account ids
            min_interactions: Filters items that have had at least d interactions
            offset: Offset to start the search used to paginate
            page_admin_top_country: Limits to only posts where the page is set to the country
            search_term: Returns posts with the search term
            sort_by: Sorting the items
            start_date: Initial date of creation of the post that will be returned in the request
            timeframe: Set end_date = end_date - timeframe
            types: Type of post to return
            verified: Whether verified accounts should be limited or excluded from return

        Returns:

        """

        payload = {'accounts': accounts, 'brandedContent': branded_content, 'count': count, 'endDate': end_date,
                   'includeHistory': include_history, 'language': language, 'listIds': list_ids,
                   'minInteractions': min_interactions, 'offset': offset, 'pageAdminTopCountry': page_admin_top_country,
                   'searchTerm': search_term, 'sortBy': sort_by, 'startDate': start_date, 'timeframe': timeframe,
                   'types': types, 'verified': verified}

        payload = {k: v for k, v in payload.items() if v}
        return payload

    def make_request_leader(self, payload=None):
        """
       Returns the request link to API
        Args:
            payload: payload used in request

        Returns: object type request

        """
        return requests.get('https://api.crowdtangle.com/leaderboard', headers={'x-api-token': self.ct_token},
                            params=payload)

    def make_request_post(self, payload=None, post_id=''):
        return requests.get(f'https://api.crowdtangle.com/post/{post_id}', headers={'x-api-token': self.ct_token},
                            params=payload)

    def make_request_posts(self, payload=None):
        return requests.get('https://api.crowdtangle.com/posts', headers={'x-api-token': self.ct_token}, params=payload)

    def get_leader(self, account_ids=None, count=None, end_date=None, list_id=None, offset=None, order_by=None,
                   sort_by=None, start_date=None):
        """

        Args:
            account_ids: ids of the accounts we want to use
            count: amount of items we want to return
            end_date: day the search should stop
            list_id: The list of the leaderboard to retrieve.
            offset: The number of rows to offset.
            order_by: the order of the sort.
            sort_by: The method by which the accountStatistics are sorted.
            start_date: The startDate of the leaderboard rage.

        Returns: organized information
        """
        payload = self.create_payload_leader(account_ids, count, end_date, list_id, offset, order_by, sort_by,
                                             start_date)
        response = self.make_request_leader(payload)
        return response.json()

    def get_post(self, include_history=None, post_id=''):
        """
        Returns the post with the id passed
        Args:
            post_id: id of the post
            include_history: Boolean to include or exclude history data

        Returns: organized information

        """
        payload = {'includeHistory': include_history}
        response = self.make_request_post(payload=payload, post_id=post_id)
        print(respose)
        return response.json()

    def get_posts(self, accounts=None, branded_content=None, count=None, end_date=None, include_history=None,
                  language=None, list_ids=None, min_interactions=None, offset=None, page_admin_top_country=None,
                  search_term=None, sort_by=None, start_date=None, timeframe=None, types=None, verified=None):
        """

        Args:
            accounts: Account nickname string or platform id
            branded_content: Limits or excludes branded content items
            count: Number of items returned
            end_date: End date the post was created
            include_history: Returns data with time series
            language: Code of the source language of the post
            list_ids: Id that contains a set of account ids
            min_interactions: Filters items that have had at least d interactions
            offset: The offset of the return item used for paging
            page_admin_top_country: Limits to only posts where the page is set to the country
            search_term: Returns posts with the search term
            sort_by: Sorting the items
            start_date: Initial date of creation of the post that will be returned in the request
            timeframe: Set end_date = end_date - timeframe
            types: Type of post to return
            verified: Whether verified accounts should be limited or excluded from return

        Returns:

        """
        # Define os valores das variáveis
        payload = self.create_payload_posts(accounts, branded_content, count, end_date, include_history, language,
                                           list_ids, min_interactions, offset, page_admin_top_country, search_term,
                                           sort_by, start_date, timeframe, types, verified)
        # Faz o request com os valores definidos
        response = self.make_request_posts(payload)
        return response.json()



from distutils.core import setup
from setuptools import find_packages
import os

current_directory = os.path.dirname(os.path.abspath(__file__))
try:
    with open(os.path.join(current_directory, 'README.md'), encoding='utf-8') as f:
        long_description = f.read()
except Exception:
    long_description = ''

setup(
    # Project name:
    name='onipkg_api',
    # Packages to include in the distribution:
    packages=find_packages(where="src"),
    package_dir={'': 'src'},
    # Project version number:
    version='1.0.23',
    # List a license for the project, eg. MIT License
    license='',
    # Short description of your library:
    description='Helper para consumo de APIs',
    # Long description of your library:
    long_description=long_description,
    long_description_content_type='text/markdown',
    # Your name:
    author='Lucas Heilbuth Nazareth de Sousa',
    # Your email address:
    author_email='lucasheilbuth@yahoo.com.br',
    # Link to your github repository or website:
    url='https://github.com/LucasHeilbuth',
    # Download Link from where the project can be downloaded from:
    download_url='https://github.com/Onimusic/oni_api_helper.git',
    # List of keywords:
    keywords=['onimusic'],
    # List project dependencies:
    install_requires=[
        'cachetools',
        'certifi',
        'charset-normalizer',
        'google-api-core',
        'google-api-python-client',
        'google-auth',
        'google-auth-httplib2',
        'googleapis-common-protos',
        'httplib2',
        'idna',
        'numpy',
        'pandas',
        'protobuf',
        'pyasn1',
        'pyasn1-modules',
        'pyparsing',
        'python-dateutil',
        'pytz',
        'requests',
        'rsa',
        'six',
        'uritemplate',
        'urllib3',
    ],
    # https://pypi.org/classifiers/
    classifiers = [
    "Programming Language :: Python :: 3",
    "License :: OSI Approved :: MIT License",
    "Operating System :: OS Independent",
]
)



import base64
import datetime
import gzip
import pandas as pd
import requests
import json


class SpotifyCredential:
    access_token = None
    access_token_expires = datetime.datetime.now()
    access_token_did_expire = True
    need_access = False
    range_api_success = [200, 201, 202, 203, 204, 205, 206, 207, 208, 226]
    token_url = "https://accounts.spotify.com/api/token"

    def __init__(self, client_id, client_secret):
        """
        Função que recebe os parametros necessários para a autenticação
        Args:
            client_id: Credential id client
            client_secret: Credential secret client
        """
        self.client_id = client_id
        self.client_secret = client_secret

    def get_client_credentials(self):
        """
        Recebe o client id e os client secrets e funciona para encodificar e decodificar as credenciais
        Returns: retorna as credenciais codificadas em base64

        """
        if self.client_secret is None or self.client_id is None:
            raise Exception("You must set client_id and client_secret")
        client_creds = f"{self.client_id}:{self.client_secret}"
        client_creds_b64 = base64.b64encode(client_creds.encode())
        return client_creds_b64.decode()

    def get_token_headers(self):
        """
        Utiliza as credenciais encodificadas com base 64 e nos retorna o header com as credenciais no formato que
        precisamos
        Returns: Header que possui a credencial encodificada em base 64

        """
        client_creds_b64 = self.get_client_credentials()
        return {"Authorization": f"Basic {client_creds_b64}"}

    @staticmethod
    def get_token_data() -> dict:
        """

        Returns: grant_type da credencial.

        """
        return {"grant_type": "client_credentials"}

    def perform_auth(self):
        """
        Realiza o processo de autenticação
        Returns: True em caso de sucesso

        """
        token_url = self.token_url
        token_data = self.get_token_data()
        token_headers = self.get_token_headers()
        r = requests.post(token_url, data=token_data, headers=token_headers, allow_redirects=True, verify=True)
        if r.status_code not in range(200, 299):
            raise Exception("Could not authenticate client.")
            # return False
        data = r.json()
        now = datetime.datetime.now()
        access_token = data['access_token']
        expires_in = data['expires_in']  # seconds
        expires = now + datetime.timedelta(seconds=expires_in)
        self.access_token = access_token
        self.access_token_expires = expires
        self.access_token_did_expire = expires < now
        return True

    def get_access_token(self):
        """
        Utiliza o access token verificando a checagem se o mesmo está expirado
        Returns: Token de autenticação.

        """
        token = self.access_token
        expires = self.access_token_expires
        now = datetime.datetime.now()
        if expires < now or token is None:
            self.perform_auth()
            return self.get_access_token()
        return token

    def get_resource_header(self):
        """
        Utiliza a função get_access_token para pegar o token e o coloca em um formato de header para utilização
        Returns: Header no formato para utilização.

        """
        if not self.need_access:
            access_token = self.get_access_token()
            return {"Authorization": f"Bearer {access_token}"}
        raise Exception('Need Access Token')


class SpotifyPublic(SpotifyCredential):
    """
    Classe que faz todas as funções relacionadas com a API do spotify, desde autenticação até a chamada do API
    """
    def __init__(self, client_id, client_secret):
        """
        Função que recebe os parametros necessários para a autenticação
        Args:
            client_id: Credential id client
            client_secret: Credential secret client
        """
        super().__init__(client_id, client_secret)

    @staticmethod
    def get_resource(lookup_id, resource_type='', version='v1', api_lookup_type=None, extra_filter=None, market=None,
                     search_type=None):
        """
        Recebe os parametros que utilizaremos na metodo de request da api e nos
        redireciona para qual metodo que queremos
        Args:
            search_type:
            lookup_id: id que sera utilizado na busca
            resource_type: tipo de busca que queremos fazer
            version: versão da api que utilizaremos (v1 como padrão)
            api_lookup_type: tipo de pesquisa que queremos fazer se o resource type for algo generico, utilizado
            na busca de playlists como o resource_type é user
            extra_filter: filtro extra caso necessário, utilizar como disponivel no site do api do spotify
            market: mercado em que queremos fazer a busca (Ex:BR)

        Returns: Dict com as informações que foram requesitadas pelo metodo em que a função foi utilizada

        """
        endpoint = f"https://api.spotify.com/{version}/{resource_type}/{lookup_id}"
        if api_lookup_type is not None:
            endpoint = f"https://api.spotify.com/{version}/{resource_type}/{lookup_id}/{api_lookup_type}"
        if market is not None:
            endpoint = f"https://api.spotify.com/{version}/{resource_type}/{lookup_id}/{api_lookup_type}?{market}"
        if extra_filter is not None:
            endpoint = f"https://api.spotify.com/{version}/{resource_type}/{lookup_id}/{api_lookup_type}{extra_filter}"
        if search_type is not None:
            if resource_type:
                endpoint = f"https://api.spotify.com/{version}/{resource_type}/{lookup_id}?{search_type}"
                if api_lookup_type:
                    endpoint = f"https://api.spotify.com/{version}/{resource_type}/{lookup_id}/{api_lookup_type}?{search_type}"
            else:
                endpoint = f"https://api.spotify.com/{version}/{lookup_id}?{search_type}"
        return endpoint

    def get_artist_top_tracks(self, artist_id, offset=0):
        """

        Args:
            artist_id: id de um artista
            offset: Initial position of the items to return.

        Returns: top tracks desse artista

        """
        endpoint = self.get_resource(resource_type='artists', lookup_id=artist_id,
                                     api_lookup_type=f'top-tracks?market=br&limit=50&offset={offset}')
        header = self.get_resource_header()
        return requests.get(endpoint, headers=header).json()

    def get_playlist(self, playlist_id, offset=0):
        """

        Args:
            playlist_id: id da playlist
            offset: Initial position of the items to return.

        Returns: Json with info of tracks from the playlist

        """
        endpoint = self.get_resource(resource_type='playlists', lookup_id=playlist_id,
                                     search_type=f'limit=50&offset={offset}')
        header = self.get_resource_header()
        return requests.get(endpoint, headers=header).json()

    def get_search_artist_type_playlist(self, artist, offset=0):
        """

        Args:
            artist: nome do artista
            offset: Initial position of the items to return.

        Returns: playlist editoriais do artista

        """
        endpoint = self.get_resource(lookup_id='search', search_type=f'q="artist:{artist}"&type=playlist&offset='
                                                                     f'{offset}&limit=50')
        header = self.get_resource_header()
        return requests.get(endpoint, headers=header).json()

    def get_categories(self, offset=0):
        """
        Função que retorna as categorias disponíveis na api do spotify
        Args:
            offset: Initial position of the items to return.
        Returns: List of categories

        """
        endpoint = self.get_resource(resource_type='browse', lookup_id='categories',
                                     search_type=f'limit=50&offset={offset}')
        header = self.get_resource_header()
        return requests.get(endpoint, headers=header).json()

    def get_info_endpoint(self, endpoint):
        header = self.get_resource_header()
        return requests.get(endpoint, headers=header).json()

    def get_single_browse_category(self, category_id, offset=0):
        """
        Get info of the playlist of a category
        Args:
            category_id: id da categoria
            offset: Initial position of the items to return.

        Returns: playlists da categoria

        """
        endpoint = self.get_resource(resource_type='browse', lookup_id='categories',
                                     api_lookup_type=f'{category_id}/playlists', search_type=f'limit=50&offset={offset}'
                                     )
        header = self.get_resource_header()
        return requests.get(endpoint, headers=header).json()

    def get_featured_playlists(self, timestamp=None, offset=0):
        """
        Get info of the featured playlists
        Args:
            timestamp: timestamp of the date that you want to get the featured playlists
            offset: Initial position of the items to return.
        Returns: playlists em destaque

        """
        endpoint = self.get_resource(resource_type='browse', lookup_id='featured-playlists',
                                     search_type=f'timestamp={timestamp}&limit=50&offset={offset}')
        header = self.get_resource_header()
        return requests.get(endpoint, headers=header).json()

    def get_users_playlists(self, user_id, offset=0):
        """

        Args:
            user_id: id do usuario
            offset: Initial item returned

        Returns: playlists do usuario

        """
        endpoint = self.get_resource(resource_type='users', lookup_id=user_id,
                                     api_lookup_type='playlists', search_type=f'limit=50&offset={offset}')
        header = self.get_resource_header()
        return requests.get(endpoint, headers=header).json()

    def get_track(self, track_id):
        """
        Get info of the playlist of a track
        Args:
            track_id: id do track

        Returns: playlists que contém o track

        """
        endpoint = self.get_resource(resource_type='tracks', lookup_id=track_id, search_type='limit=50')
        header = self.get_resource_header()
        return requests.get(endpoint, headers=header).json()

    def get_several_tracks(self, track_ids):
        """
        Get info about several tracks
        Args:
            track_ids: list of track ids
        Returns: Dict with info of tracks
        """
        endpoint = self.get_resource(lookup_id='tracks', search_type=f'ids={track_ids}&limit=50')
        header = self.get_resource_header()
        return requests.get(endpoint, headers=header).json()

    def get_several_artists(self, artist_id):
        """
        Get info about several artists
        Args:
            artist_id: list of artist ids
        Returns: Dict with info of artists
        """
        endpoint = self.get_resource(lookup_id='artists', search_type=f'ids={artist_id}&limit=50')
        header = self.get_resource_header()
        return requests.get(endpoint, headers=header).json()


class SpotifyPrivate(SpotifyCredential):
    """
    Classe que faz todas as funções relacionadas com a API do spotify, desde autenticação até a chamada do API Privada
    """

    def __init__(self, client_id, client_secret, licensor, access_token_const, date):
        """
        Função que recebe os parametros necessários para a autenticação
        Args:
            date: Data da requisição
            client_id:
            day: Dia da data de requisição
        """
        super().__init__(client_id, client_secret)
        self.year = date.strftime('%Y')
        self.month = date.strftime('%m')
        self.day = date.strftime('%d')
        self.header = self.get_resource_header()
        self.licensor = licensor
        self.access_token_const = access_token_const

    def get_data_streams(self, country) -> requests.models.Response:
        """
        Faz a request para o endpoint streams
        Args:
            country: Code of the country
        Returns: Respota da request
        """
        url = f'https://provider-api.spotify.com/v1/analytics/{self.licensor}/enhanced/streams/{self.year}/' \
              f'{self.month}/{self.day}/{country}'
        return requests.get(url, headers=self.header)

    def get_data_country(self) -> requests.models.Response:
        """
        Faz a request para o endpoint streams
        Returns: Respota da request
        """
        url = f'https://provider-api.spotify.com/v1/analytics/{self.licensor}/enhanced/streams/{self.year}/' \
              f'{self.month}/{self.day}'
        return requests.get(url, headers=self.header)

    def get_data_users(self) -> requests.models.Response:
        """
        Faz a request para o endpoint streams
        Returns: Respota da request
        """
        url = f'https://provider-api.spotify.com/v1/analytics/{self.licensor}/enhanced/users/{self.year}/{self.month}/'\
              f'{self.day}'
        return requests.get(url, headers=self.header)

    def get_data_tracks(self) -> requests.models.Response:
        """
        Faz a request para o endpoint tracks
        Returns: Respota da request
        """
        url = f'https://provider-api.spotify.com/v1/analytics/{self.licensor}/enhanced/tracks/{self.year}/' \
              f'{self.month}/{self.day}'
        return requests.get(url, headers=self.header)

    def get_data_aggregated_streams(self) -> requests.models.Response:
        """
        Faz a request para o endpoint aggregated streams
        Returns: Respota da request
        """
        url = f'https://provider-api.spotify.com/v1/analytics/{self.licensor}/enhanced/aggregatedstreams/{self.year}/' \
              f'{self.month}/{self.day}'
        return requests.get(url, headers=self.header)

    def get_data_sub_30_secs_streams(self, country: str) -> requests.models.Response:
        """
        Faz a request para o endpoint sub 30 secs streams
        Args:
            country: Code of the country
        Returns: Respota da request
        """
        url = f'https://provider-api.spotify.com/v1/analytics/{self.licensor}/enhanced/sub_30_sec_streams/{self.year}/' \
              f'{self.month}/{self.day}/{country}'
        return requests.get(url, headers=self.header)

    def get_full_access_header(self) -> dict:
        """
        Cria um header que possui o token com autorização, necessário para requests que necessitam de autorização
        Returns: Header com o token que possui autorização
        """
        return {"Authorization": f"Bearer {self.access_token_const}"}

    @staticmethod
    def request2json(request):
        """
        Função que transforma a resposta da request em um dataframe
        Args:
            request: request feita para o endpoint
        Returns:
            DataFrame com os dados da request
        """
        decompressed_response = gzip.decompress(request.content)
        my_json = decompressed_response.decode('utf-8')
        my_json = f'[{my_json}]'
        my_json = my_json.replace('\n', '')
        my_json = my_json.replace('}{', '},{')
        return json.loads(my_json)

    def get_tracks(self) -> pd.DataFrame:
        """
        Função que retorna o dataframe do endpoint tracks
        Args:
            type: Extensão do arquivo que vai ser escrito
        Returns:
            DataFrame tracks
        """
        return self.request2json(self.get_data_tracks())

    def get_users(self) -> pd.DataFrame:
        """
        Função que retorna o dataframe do endpoint users
        Args:
            type: Extensão do arquivo que vai ser escrito
        Returns:
            DataFrame users
        """
        return self.request2json(self.get_data_users())

    def get_streams(self, country) -> pd.DataFrame:
        """
        Função que retorna o dataframe do endpoint streams para um país específico
        Args:
            country: Code of the country
        Returns:
            DataFrame streams de um país
        """
        return self.request2json(self.get_data_streams(country))

    def get_sub_30_streams(self, country) -> pd.DataFrame:
        """
        Função que retorna o dataframe do endpoint sub30_secs_streams para um país específico
        Args:
            country: Code of the country
        Returns:
            DataFrame sub30_secs_streams streams de um país
        """
        return self.request2json(self.get_data_sub_30_secs_streams(country))

    def get_aggregated_streams(self) -> pd.DataFrame:
        """
        Função que retorna o dataframe do endpoint aggregated_streams
        Returns:
            DataFrame aggregated_streams
        """
        return self.request2json(self.get_data_aggregated_streams())
